﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SQLite;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace LiteMap
{
    class DbTools
    {

        private static string _shotTable = "CREATE TABLE [Shots] (" +
            "[Id] INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL UNIQUE," +
            "[geom] BLOB NULL," +
            "[LineName] INT NULL," +
            "[Name] INT NULL," +
            "[Patch] INT NULL," +
            "[SurveyId] INT NULL," +
            "[PropertyId] INT NULL);";

        public static bool CreateDb(string fileName)
        {
            try
            {
                SQLiteConnection.CreateFile(fileName);
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Epic fail", 
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                return false;
            }
            
            return true;
        }

        public static bool CreateTables(string conn)
        {
            List<string> sqlCmd = new List<string>();

            // Tables
            sqlCmd.Add(_shotTable);

            // Indexes
            
            // Foreign Kets
            
            SQLiteConnection myConn = new SQLiteConnection(conn);
            foreach (string table in sqlCmd)
            {

                try
                {
                    myConn.Open();
                    SQLiteCommand sqlQuery = new SQLiteCommand(table, myConn);
                    SQLiteDataReader reader = sqlQuery.ExecuteReader();                    
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString(), "Epic fail", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                finally
                {
                    if (myConn.State == ConnectionState.Open)
                    {
                        myConn.Close();
                    }
                }
            }

            // Create ThinkGeo Spatial Indexes
            myConn.Open();
            SqliteFeatureSource.CreateSpatialIndex(myConn, "Shots", "geom", 32615);
            myConn.Close();


            Debug.Print("Tables Created Successfully", "Success",
                MessageBoxButtons.OK, MessageBoxIcon.Information);

            return true;
        }

        public static bool SqlBulkInsert(DataTable table, string conn, string commandText)
        {

            var s = Stopwatch.StartNew();

            try
            {
                // Batch add records using transaction method
                using (SQLiteConnection connection = new SQLiteConnection(conn))
                {
                    connection.Open();

                    using (var transaction = connection.BeginTransaction())
                    {
                        using (var cmd = connection.CreateCommand())
                        {
                            cmd.CommandText = commandText;
                            cmd.CommandType = CommandType.Text;

                            // Map columns from the data table
                            foreach (DataColumn col in table.Columns)
                            {                                
                                cmd.Parameters.AddWithValue(@col.ColumnName, col.ColumnName);
                            }

                            // Add command for each data row
                            foreach (DataRow row in table.Rows)
	                        {
                                for (int i = 0; i < cmd.Parameters.Count; i++)
                                {
                                    SQLiteParameter parameter = cmd.Parameters[i];
                                    parameter.Value = row.ItemArray[i];
                                    if (row.ItemArray[i].GetType() == typeof(byte[]))
                                        parameter.DbType = DbType.Binary;

                                    //Debug.Print(string.Format("{0}, {1}, {2}", 
                                    //    parameter.ParameterName, parameter.DbType.ToString(), parameter.TypeName));
                                }
                                cmd.ExecuteNonQuery();
	                        }
                        }
                        transaction.Commit();
                    }
                }
            }
            catch (Exception ex)
            {
                //t.Rollback();
                MessageBox.Show(ex.ToString(), "Epic fail",
                    MessageBoxButtons.OK, MessageBoxIcon.Information);

                return false;
            }
            finally
            {
                Debug.Print(string.Format("Time to add {0} features was {1} seconds",
                    table.Rows.Count, s.Elapsed.TotalSeconds));
            }

            return true;
        }

    }
}
